@@ -38,7 +38,7 @@ import dataclasses
 import enum
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union
 
-from typing_extensions import Final, Literal, _AnnotatedAlias, get_args, get_origin
+from typing_extensions import Annotated, Final, Literal, get_args, get_origin
 
 from . import _strings
 
@@ -115,15 +115,11 @@ def _instantiator_from_container_type(
     if type_origin is None:
         return None
 
-    # Unwrap Final types.
-    if type_origin is Final:
-        (contained_type,) = get_args(typ)
+    # Unwrap Annotated and Final types.
+    if type_origin in (Annotated, Final):
+        contained_type = get_args(typ)[0]
         return instantiator_from_type(contained_type, type_from_typevar)
 
-    # Unwrap Annotated types.
-    if hasattr(typ, "__class__") and typ.__class__ == _AnnotatedAlias:
-        return instantiator_from_type(type_origin, type_from_typevar)
-
     # List, tuples, and sequences.
     if type_origin in (
         collections.abc.Sequence,  # different from typing.Sequence!
@@ -154,11 +150,9 @@ def _instantiator_from_container_type(
         typeset_no_ellipsis = typeset - {Ellipsis}  # type: ignore
 
         if typeset_no_ellipsis != typeset:
-            # Ellipsis: variable argument counts.
-            if len(typeset_no_ellipsis) > 1:
-                raise UnsupportedTypeAnnotationError(
-                    "When an ellipsis is used, tuples must contain only one type."
-                )
+            # Ellipsis: variable argument counts. When an ellipsis is used, tuples must
+            # contain only one type.
+            assert len(typeset_no_ellipsis) == 1
             (contained_type,) = typeset_no_ellipsis
 
             make, inner_meta = _instantiator_from_type_inner(
@@ -198,9 +192,12 @@ def _instantiator_from_container_type(
     # Optionals.
     if type_origin is Union:
         options = set(get_args(typ))
-        assert (
-            len(options) == 2 and type(None) in options
-        ), "Union must be either over dataclasses (for subparsers) or Optional"
+        if len(options) != 2 or type(None) not in options:
+            # Note that the subparsers logic happens much earlier.
+            raise UnsupportedTypeAnnotationError(
+                "Union must be either over dataclasses (for subparsers) or Optional"
+                " (Union[T, None])"
+            )
         (typ,) = options - {type(None)}
         instantiator, metadata = _instantiator_from_type_inner(
             typ, type_from_typevar, allow_sequences=True
@@ -210,10 +207,11 @@ def _instantiator_from_container_type(
     # Literals.
     if type_origin is Literal:
         choices = get_args(typ)
+        if not len(set(map(type, choices))) == 1:
+            raise UnsupportedTypeAnnotationError(
+                "All choices in literal must have the same type!"
+            )
         contained_type = type(next(iter(choices)))
-        assert all(
-            map(lambda c: type(c) == contained_type, choices)
-        ), "All choices in literal must have the same type!"
         if issubclass(contained_type, enum.Enum):
             choices = tuple(map(lambda x: x.name, choices))
         instantiator, metadata = _instantiator_from_type_inner(
@@ -226,7 +224,9 @@ def _instantiator_from_container_type(
         )
         return instantiator, dataclasses.replace(metadata, choices=choices)
 
-    return None
+    raise UnsupportedTypeAnnotationError(  # pragma: no cover
+        f"Unsupported type {typ} with origin {type_origin}"
+    )
 
 
 def _instantiator_from_type_inner(
